from .account import AccountAPI
from .auth import AuthAPI
from .author import AuthorAPI
from .chapter import ChapterAPI
from .cover import CoverAPI
from .group import GroupAPI
from .list import ListAPI
from .manga import MangaAPI
from .misc import MiscAPI
from .user import UserAPI
from .upload import UploadAPI


__all__ = (
    "AccountAPI", "AuthAPI", "AuthorAPI", "ChapterAPI", "GroupAPI", "ListAPI",
    "MangaAPI", "MiscAPI", "UserAPI", "CoverAPI", "UploadAPI"
)
